// brass.cpp -- mehtods

#include <iostream>
#include "brass.h"
using std::cout;
using std::endl;
using std::string;

//formatting stuff
typedef std::ios_base::fmtflags format;
typedef std::streamsize  precis;
format setFormat();
void restore(format f, precis p);

// Brass methods

Brass::Brass(const string & s, long an, double bal)
: fullName(s), acctNum(an), balance(bal) {}

void Brass::Deposit(double amt) {
    if(amt < 0) cout << "You can't deposit a negative.  Action cancelled.\n";
    else balance += amt;
}

void Brass::Withdraw(double amt) {
    // set up ###.## format
    format initialState = setFormat();
    precis prec = cout.precision(2);

    if(amt < 0) cout << "Withdraw amt must be positive.\n";
    else if(amt <= balance)
        balance -= amt;
    else
        cout << "Withdraw amount $" << amt <<
                " exceeds the balance in your account.\n";
    restore(initialState, prec);
}

double  Brass::Balance() const {
    return balance;
}

void Brass::ViewAcct() const {
    format initialState = setFormat();
    precis prec = cout.precision(2);
    cout << "Client: " << fullName << endl;
    cout << "Account Number: " << acctNum << endl;
    cout << "Balance: " << balance << endl;
    restore(initialState, prec); //restore original formatting
}

// BrasPlus Mehtods

BrassPlus::BrassPlus(const string & s, long an, double bal,
                     double ml, double r) : Brass(s,an,bal) {
    maxLoan = ml;
    owesBank = 0;
    rate = r;
}

BrassPlus::BrassPlus(const Brass & ba, double ml,
              double r) : Brass(ba) // uses implicit copy constructor
{
    maxLoan = ml;
    owesBank = 0.0;
    rate = r;
}

//redefine how V3iewAcct() works
void BrassPlus::ViewAcct() const {
    //set the $###.## format
    format initialState = setFormat();
    precis prec = cout.precision(2);

    Brass::ViewAcct(); //display base portion
    cout << "Maximum loan amount: $" << maxLoan << endl;
    cout << "Owed to bank: $" << owesBank << endl;
    cout.precision(3);
    cout << "Rate: " << 100*rate << "%\n";
    restore(initialState, prec);
}

// Redefine how Withdraw Works:
void BrassPlus::Withdraw(double amt) {
    // set up the $###.## firnat
    format initialState = setFormat();
    precis prec = cout.precision(2);
    if(amt < 0)  cout << "Withdrawal must be positive. "
                    << " Withdrawal cancelled.\n";

    //double bal = Balance();  // balance protected not private
    if(amt <= balance) balance -= amt;
    else if (amt <= balance + maxLoan - owesBank) {
        double advance = amt - balance;
        owesBank += advance*(1.0+rate);
        cout << "Bank Advance: $" << advance << endl;
        cout << "Finance Charge: $" << advance * rate << endl;
        Deposit(advance);
        Brass::Withdraw(amt);
    }
    else cout << "Credit limit exceeded. Transaction cancelled.\n";
    restore(initialState,prec);
}

format setFormat() {
    // set up ###.## firmat
    return cout.setf(std::ios_base::fixed, std::ios_base::floatfield);
}

void restore(format f, precis p) {
    cout.setf(f, std::ios_base::floatfield);
    cout.precision(p);
}
